package com.cnbm.basic.controller;

import com.cnbm.admin.annotation.LogOperation;
import com.cnbm.admin.params.IdParam;
import com.cnbm.basic.dto.*;
import com.cnbm.basic.entity.ProductFeatures;
import com.cnbm.common.constant.Constant;
import com.cnbm.common.page.PageData;
import com.cnbm.common.utils.ExcelUtils;
import com.cnbm.common.utils.Result;
import com.cnbm.common.validator.AssertUtils;
import com.cnbm.common.validator.ValidatorUtils;
import com.cnbm.common.validator.group.AddGroup;
import com.cnbm.common.validator.group.DefaultGroup;
import com.cnbm.common.validator.group.UpdateGroup;
import com.cnbm.basic.excel.ProductFeaturesExcel;
import com.cnbm.basic.service.IProductFeaturesService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 产品特性 表  前端控制器
 *
 * @author why
 * @since  2022-07-15
 */
@RestController
@RequestMapping("/basic/productFeatures")
@Api(tags="产品特性 表")
public class ProductFeaturesController {
    @Autowired
    private IProductFeaturesService productFeaturesService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataTypeClass=Integer.class) ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataTypeClass=String.class) ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataTypeClass=String.class),
        @ApiImplicitParam(name = "id", value = "id", paramType = "query", dataTypeClass = Long.class)
    })
    @PreAuthorize("@ex.hasAuthority('basic:productFeatures:page')")
    public Result<PageData<ProductFeaturesDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<ProductFeaturesDTO> page = productFeaturesService.page(params);

        return new Result<PageData<ProductFeaturesDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @PreAuthorize("@ex.hasAuthority('basic:productFeatures:info')")
    public Result<ProductFeaturesDTO> get(@PathVariable("id") Long id){
        ProductFeaturesDTO data = productFeaturesService.get(id);

        return new Result<ProductFeaturesDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @PreAuthorize("@ex.hasAuthority('basic:productFeatures:save')")
    public Result save(@RequestBody ProductFeaturesDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        productFeaturesService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @PreAuthorize("@ex.hasAuthority('basic:productFeatures:update')")
    public Result update(@RequestBody ProductFeaturesDTO dto){
        //效验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        productFeaturesService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @PreAuthorize("@ex.hasAuthority('basic:productFeatures:delete')")
    public Result delete(@RequestBody Long[] ids){
        //效验数据
        AssertUtils.isArrayEmpty(ids, "id");

        productFeaturesService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "productId", value = "产品", paramType = "query", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "inspectionStage", value = "检测阶段", paramType = "query", dataTypeClass = Integer.class)
    })
    @PreAuthorize("@ex.hasAuthority('basic:productFeatures:export')")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<ProductFeaturesDTO> list = productFeaturesService.getFeaturesList(params);

        ExcelUtils.exportExcelToTarget(response, "ProductFeatures", list, ProductFeaturesExcel.class);
    }

    @PostMapping("getProductFeaturesByProductId")
    @ApiOperation("查询产品特性 依据产品id")
    public Result<List<ProductFeaturesDTO>> getProductFeaturesByProductId(@RequestBody IdParam id){
        List<ProductFeaturesDTO> list = productFeaturesService.getProductFeaturesByProductId(id.getId());
        return new Result<List<ProductFeaturesDTO>>().ok(list);
    }

    @PostMapping("getFeaturesByStageProcedure")
    @ApiOperation("查询依据工序id 检验阶段")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "productId", value = "产品", paramType = "query", dataTypeClass = Long.class),
            //@ApiImplicitParam(name = "workingProcedureId", value = "工序", paramType = "query", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "inspectionStage", value = "检测阶段", paramType = "query", dataTypeClass = Integer.class)
    })
    public Result<List<FeaturesProcedureDTO>> getFeaturesByStageProcedure(@ApiIgnore @RequestParam Map<String, Object> params){
        List<FeaturesProcedureDTO> list = productFeaturesService.getFeaturesByStageProcedure(params);
        return new Result<List<FeaturesProcedureDTO>>().ok(list);
    }

    @PutMapping("batchUpdate")
    @ApiOperation("批量更新特性")
    public Result batchUpdate(@RequestBody ProductFeaturesDTO[] lists){

        productFeaturesService.batchUpdate(lists);

        return new Result();
    }

    @PostMapping("getFeaturesList")
    @ApiOperation("特性列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "productId", value = "产品", paramType = "query", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "inspectionStage", value = "检测阶段", paramType = "query", dataTypeClass = Integer.class)
    })
    public Result<List<ProductFeaturesDTO>> getFeaturesList(@ApiIgnore @RequestParam Map<String, Object> params){
        List<ProductFeaturesDTO> list = productFeaturesService.getFeaturesList(params);
        return new Result<List<ProductFeaturesDTO>>().ok(list);
    }

    @PostMapping(value = "list")
    @ApiOperation(value = "获取特性列表")
    @LogOperation("获取特性列表")
    public List<ProductFeaturesDTO> list() {
        return productFeaturesService.list();
    }

}